readme = '''
# [PACKAGE_NAME]

[SHIELDS]

## Description

[DESCRIPTION]

## Install

~~~~bash
pip install [PACKAGE_NAME]
# or
pip3 install [PACKAGE_NAME]
~~~~

## Usage

~~~~python
import [PACKAGE_NAME]
~~~~

## Dependencies

[DEPENDENCIES]
'''.strip()
